from django.contrib.auth.models import Group, User
from django.test import TestCase

from django_spire.auth.mfa.models import MfaCode
from django_spire.auth.user.tests.factories import create_super_user
from django_spire.contrib.service.tests.services import TestUserModelService, TestMfaCodeModelService


class TestInstanceService(TestCase):
    def setUp(self):
        User.services = TestUserModelService()
        MfaCode.services = TestMfaCodeModelService()

        self.user = create_super_user()
        self.group = Group.objects.create(name='Boberts Minions')

    def tearDown(self):
        User.services = None
        MfaCode.services = None

    def test_valid_update_model_fields(self):
        data = {
            'first_name': 'John',
            'last_name': 'Smith'
        }

        created = self.user.services.model_obj_validate_field_data_and_save(**data)
        self.assertEqual(self.user.first_name, 'John')
        self.assertEqual(self.user.last_name, 'Smith')
        self.assertFalse(created)

    def test_valid_create_model_fields(self):
        new_user = User()

        data = {
            'first_name': 'John',
            'last_name': 'Smith'
        }

        created = new_user.services.model_obj_validate_field_data_and_save(**data)
        self.assertEqual(new_user.first_name, 'John')
        self.assertEqual(new_user.services.user.first_name, 'John')
        self.assertEqual(new_user.last_name, 'Smith')
        self.assertEqual(new_user.services.user.last_name, 'Smith')
        self.assertIsNotNone(self.user.id)
        self.assertTrue(created)

    def test_invalid_field_name(self):
        # Skips the field and saves the instance.
        data = {
            'invalid_field': 'test'
        }
        created = self.user.services.model_obj_validate_field_data_and_save(**data)
        self.assertFalse(created)

    def test_foreign_key_id_aliases(self):
        mfa = MfaCode.objects.create(
            user=self.user,
            code='123456',
            expiration_datetime='2023-01-01'
        )
        new_user = create_super_user()

        data = {
            'user_id': new_user.id,
        }

        created = mfa.services.model_obj_validate_field_data_and_save(**data)
        self.assertEqual(mfa.user_id, new_user.id)
        self.assertFalse(created)

    def test_obj_foreign_key_id_aliases(self):
        mfa = MfaCode.objects.create(
            user=self.user,
            code='123456',
            expiration_datetime='2023-01-01'
        )
        new_user = create_super_user()

        data = {
            'user': new_user,
        }

        created = mfa.services.model_obj_validate_field_data_and_save(**data)
        self.assertEqual(mfa.user_id, new_user.id)
        self.assertFalse(created)



from django.contrib.auth.models import User
from django.test import TestCase

from django_spire.auth.user.tests.factories import create_super_user
from django_spire.contrib.service.exceptions import ServiceException
from django_spire.contrib.service.django_model_service import BaseDjangoModelService
from django_spire.contrib.service.tests.services import TestUserModelService


class TestBaseService(TestCase):
    def setUp(self):
        User.services = TestUserModelService()

        self.user = create_super_user()

    def tearDown(self):
        User.services = None

    def test_abstraction_on_init(self):
        with self.assertRaises(ServiceException) as context:
            class BrokenModelService(BaseDjangoModelService):
                def create_taco(self):
                    return "Taco!"

            User.services = BrokenModelService()

            self.user.services.create_taco()

    def test_model_obj_is_created(self):
        self.assertTrue(self.user.services.model_obj_is_created)
        self.assertFalse(User().services.model_obj_is_created)

    def test_model_obj_is_new(self):
        new_user = User()

        self.assertTrue(new_user.services.model_obj_is_new)
        self.assertFalse(self.user.services.model_obj_is_new)



import base64
import uuid

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from django_spire.notification.sms.models import SmsTemporaryMedia


@csrf_exempt
def external_temporary_media_view(request, external_access_key: uuid.UUID) -> HttpResponse:
    temporary_media = SmsTemporaryMedia.objects.get(external_access_key=external_access_key)

    return HttpResponse(
        content=base64.b64decode(temporary_media.content),
        content_type=temporary_media.content_type
    )



from __future__ import annotations

from abc import ABC
from typing import Any

from django_spire.contrib.service.exceptions import ServiceException


class BaseService(ABC):
    def __init__(self, obj: Any = None):
        self._obj_name: str = ...
        self._obj_type: type = ...

        if ABC not in self.__class__.__bases__:
            self._set_obj_from_annotations(obj)

            if not self._obj_is_valid:
                raise ServiceException(f'{self._obj_name} failed to validate on {self.__class__.__name__}')

    def __init_subclass__(cls):
        super().__init_subclass__()

        if ABC not in cls.__bases__:
            cls._validate_annotations_or_error()

            # Typing Does not work properly for services if you override __get__ in the BaseService class.
            # This is a workaround and should be fixed in future versions of the python lsp.
            def __get__(self, instance, owner):
                if instance is None:
                    target: BaseService | Any = owner()
                else:
                    target: BaseService | Any = instance

                if issubclass(target.__class__, BaseService):

                    self._validate_target_or_error(target)

                    return cls(getattr(target, self._obj_name))

                return cls(target)

            setattr(cls, '__get__', __get__)

    @property
    def obj(self) -> Any:
        return getattr(self, self._obj_name)

    @property
    def _obj_is_valid(self) -> bool:
        return isinstance(self.obj, self._obj_type)

    def _set_obj_from_annotations(self, obj: Any = None):
        for obj_name, obj_type in self.__class__.__annotations__.items():
            if not issubclass(obj_type, BaseService):
                self._obj_name = obj_name
                self._obj_type = obj_type
                break

        if self._obj_type is None:
            raise ServiceException(
                f'{self.__class__.__name__} annotated class attribute cannot be type None.')

        if obj is None:
            setattr(self, self._obj_name, self._obj_type())
        else:
            setattr(self, self._obj_name, obj)

    @classmethod
    def _validate_annotations_or_error(cls):
        non_base_service_objects_count = 0

        for obj_name, obj_type in cls.__annotations__.items():
            if not issubclass(obj_type, BaseService):
                non_base_service_objects_count += 1

        if non_base_service_objects_count != 1:
            raise ServiceException(
                f'{cls.__name__} must have exactly one non-BaseService annotated class attribute. Found {non_base_service_objects_count}'
            )

    def _validate_target_or_error(self, target: BaseService | Any):
        if target._obj_name != self._obj_name:
            raise ServiceException(
                f'{target.__class__.__name__} is required to have the same obj name as {self.__class__.__name__}. "{target._obj_name}" is not "{self._obj_name}".')

        if target._obj_type != self._obj_type:
            raise ServiceException(
                f'{target.__class__.__name__} must use the same obj type as {self.__class__.__name__}. {target._obj_type} is not {self._obj_type}.')



from django.contrib.auth.models import User

from django_spire.auth.mfa.models import MfaCode
from django_spire.contrib.service.django_model_service import BaseDjangoModelService


class TestUserSubModelService(BaseDjangoModelService):
    user: User

    def get_the_full_name(self):
        return self.user.get_full_name()


class TestUserModelService(BaseDjangoModelService):
    user: User
    sub: TestUserSubModelService = TestUserSubModelService()

    def get_the_first_name(self, weather: str = ''):
        return self.user.first_name + weather


class TestMfaCodeModelService(BaseDjangoModelService):
    mfa_code: MfaCode

    def get_code_length(self):
        return len(self.mfa_code.code)


from django.contrib.auth.models import Group, User
from django.test import TestCase

from django_spire.auth.mfa.models import MfaCode
from django_spire.auth.user.tests.factories import create_super_user
from django_spire.contrib.service.tests.services import TestUserModelService, TestMfaCodeModelService


class TestSubService(TestCase):
    def setUp(self):
        User.services = TestUserModelService()
        MfaCode.services = TestMfaCodeModelService()

        self.user = create_super_user()
        self.group = Group.objects.create(name='Boberts Minions')

    def tearDown(self):
        User.services = None
        MfaCode.services = None

    def test_sub_service_access(self):
        data = {
            'first_name': 'John',
            'last_name': 'Smith'
        }

        created = self.user.services.model_obj_validate_field_data_and_save(**data)

        self.assertEqual(self.user.services.sub.get_the_full_name(), 'John Smith')

